-- author: David Bartunek
-- Modul : Prevedeni std_logic_vectoru na BNC kod k ovladani sedmisegmentovek
-- Vstup    :   clk   	            -> hodiny (100Mhz)
--              dd_i_Start          -> log.1 pokud chci prevest data
--              dd_i_Binary         -> vstupni binarni data
--
-- Vystup   :   dd_o_DV             -> log.1 pokud jsou vylidni data na vystupu    
--              dd_seg0_BNC_out     -> vystupni data na hodnota BNC
--              dd_seg1_BNC_out     -> vystupni data na hodnota BNC
--              dd_seg2_BNC_out     -> vystupni data na hodnota BNC
--              dd_seg3_BNC_out     -> vystupni data na hodnota BNC
--              dd_seg4_BNC_out     -> vystupni data na hodnota BNC
--              dd_seg5_BNC_out     -> vystupni data na hodnota BNC
--              dd_seg6_BNC_out     -> vystupni data na hodnota BNC

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
 
entity double_dabble is
  port (
    --double dabble
    clk               : in std_logic;
    dd_i_Start        : in std_logic := '1';
    dd_i_Binary       : in std_logic_vector(23 downto 0);
     
    dd_o_DV           : out std_logic;
    dd_seg0_BNC_out   : out std_logic_vector(3 downto 0);
    dd_seg1_BNC_out   : out std_logic_vector(3 downto 0);
    dd_seg2_BNC_out   : out std_logic_vector(3 downto 0);
    dd_seg3_BNC_out   : out std_logic_vector(3 downto 0);
    dd_seg4_BNC_out   : out std_logic_vector(3 downto 0);
    dd_seg5_BNC_out   : out std_logic_vector(3 downto 0);
    dd_seg6_BNC_out   : out std_logic_vector(3 downto 0)

    );
end entity;
 
architecture rtl of double_dabble is
 
  type t_BCD_State is (s_IDLE, s_SHIFT, s_CHECK_SHIFT_INDEX, s_ADD,
                       s_CHECK_DIGIT_INDEX, s_BCD_DONE);
  signal r_SM_Main : t_BCD_State := s_IDLE;

  signal r_BCD : std_logic_vector(27 downto 0) := (others => '0');
  signal r_Binary : std_logic_vector(23 downto 0) := (others => '0');
  signal r_Digit_Index : natural range 0 to 6 := 0;
  signal r_Loop_Count : natural range 0 to 23  := 0;
  signal dd_o_BCD :  std_logic_vector(27 downto 0);
begin
  Double_Dabble : process (clk)
    variable v_Upper     : natural;
    variable v_Lower     : natural;
    variable v_BCD_Digit : unsigned(3 downto 0);
	 
  begin
    if rising_edge(clk) then
 
      case r_SM_Main is
        when s_IDLE =>
          if dd_i_Start = '1' then
            r_BCD     <= (others => '0');
            r_Binary  <= dd_i_Binary;
            r_SM_Main <= s_SHIFT;
          else
            r_SM_Main <= s_IDLE;
          end if;

        when s_SHIFT =>
          r_BCD     <= r_BCD(r_BCD'left-1 downto 0) & r_Binary(r_Binary'left);
          r_Binary  <= r_Binary(r_Binary'left-1 downto 0) & '0';
          r_SM_Main <= s_CHECK_SHIFT_INDEX;

        when s_CHECK_SHIFT_INDEX => 
          if r_Loop_Count = 23 then
            r_Loop_Count <= 0;
            r_SM_Main    <= s_BCD_DONE;
          else
            r_Loop_Count <= r_Loop_Count + 1;
            r_SM_Main    <= s_ADD;
          end if;

        when s_ADD =>
          v_Upper     := r_Digit_Index*4 + 3;
          v_Lower     := r_Digit_Index*4;
          v_BCD_Digit := unsigned(r_BCD(v_Lower + 3 downto v_Lower));
           
          if v_BCD_Digit > 4 then
            v_BCD_Digit := v_BCD_Digit + 3;
          end if;
 
          r_BCD(v_Upper downto v_Lower) <= std_logic_vector(v_BCD_Digit);
          r_SM_Main <= s_CHECK_DIGIT_INDEX;

        when s_CHECK_DIGIT_INDEX =>
          if r_Digit_Index = 6 then
            r_Digit_Index <= 0;
            r_SM_Main     <= s_SHIFT;
          else
            r_Digit_Index <= r_Digit_Index + 1;
            r_SM_Main     <= s_ADD;
          end if;
 
 
        when s_BCD_DONE =>
          r_SM_Main <= s_IDLE;
          dd_seg0_BNC_out <= dd_o_BCD(3 downto 0); 
          dd_seg1_BNC_out <= dd_o_BCD(7 downto 4); 
          dd_seg2_BNC_out <= dd_o_BCD(11 downto 8); 
          dd_seg3_BNC_out <= dd_o_BCD(15 downto 12); 
          dd_seg4_BNC_out <= dd_o_BCD(19 downto 16); 
          dd_seg5_BNC_out <= dd_o_BCD(23 downto 20); 
          dd_seg6_BNC_out <= dd_o_BCD(27 downto 24); 
        when others =>
          r_SM_Main <= s_IDLE;
           
      end case;
    end if;                             -- rising_edge(clk)
  end process Double_Dabble;

  dd_o_DV  <= '1' when r_SM_Main = s_BCD_DONE else '0';
  dd_o_BCD <= r_BCD;
   
end architecture rtl;